/**
 * @libdoc Cervisia Class Overview
 *
 * This is the class documentation for the Cervisia CVS front end. At the moment,
 * the only way this code can easily be used in other applications is via the
 * KPart. Hopefully it should be possible to provide more flexible facilities in
 * future versions.
 *
 * @li @ref CervisiaPart
 * A KPart that provides a reusable way to embed the Cervisia interface in an app.
 *
 * @li @ref CervisiaShell
 * A basic main window shell that embeds the CervisiaPart to provide the standalone
 * Cervisia application.
 */

//krazy:skip
